"""
Django settings for smash project.

Generated by 'django-admin startproject' using Django 1.10.3.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

import functools
import os

from django.contrib.staticfiles import storage

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
PROJECT_PATH = os.path.abspath(os.path.dirname(__file__))
# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

DEBUG = True

WSGI_APPLICATION = 'smash.wsgi.application'

SERVE_STATIC = False

# whether 2 steps authentication is mandatory to access the system
FORCE_2FA = False

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django_cleanup',
    'django_cron',
    'django_otp',
    'django_otp.plugins.otp_static',
    'django_otp.plugins.otp_totp',
    'two_factor',
    'web',
    'stronghold',

    'debug_toolbar'
]

MIDDLEWARE = [
    'web.middleware.PrivacyNoticeMiddleware',
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'whitenoise.middleware.WhiteNoiseMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django_otp.middleware.OTPMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'stronghold.middleware.LoginRequiredMiddleware',
]

ROOT_URLCONF = 'smash.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(PROJECT_PATH, '../web', 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'smash.context_processors.login_background'
            ],
        },
    },
]

CRON_CLASSES = [
    'web.views.kit.KitRequestEmailSendJob',
    'web.views.virus_mail.KitRequestEmailSendJob',
    'web.redcap_connector.RedCapRefreshJob',
    'web.views.voucher.ExpireVouchersJob',
    'web.importer.importer_cron_job.SubjectImporterCronJob',
    'web.importer.importer_cron_job.VisitImporterCronJob'
]

# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

INTERNAL_IPS = [
    '127.0.0.1'
]
STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'npm.finders.NpmFinder',
]

NPM_FILE_PATTERNS = {
    'admin-lte': ['dist/*'],
    'awesomplete': ['awesomplete.css', 'awesomplete.css.map', 'awesomplete.min.js', 'awesomplete.min.js.map'],
    'bootstrap': ['dist/*'],
    'bootstrap-datepicker': ['dist/*'],
    'cookieconsent': ['build/*'],
    'datatables.net': ['js/*'],
    'datatables.net-bs': ['css/*', 'js/*'],
    'datatables.net-buttons': ['js/*'],
    'datatables.net-buttons-bs': ['js/*', 'css/*'],
    'eonasdan-bootstrap-datetimepicker': ['build/js/*', 'build/css/bootstrap-datetimepicker.*'],
    'font-awesome': ['css/*', 'fonts/*', 'less/*', 'scss/*'],
    'fullcalendar': ['dist/*'],
    'fullcalendar-scheduler': ['dist/*'],
    'html2canvas': ['dist/*'],
    'icheck': ['skins/*', 'icheck*'],
    'ionicons': ['css/*', 'fonts/*', 'less/*', 'png/*', 'scss/*'],
    'jquery': ['dist/*'],
    'jquery-ui-dist': ['jquery-ui*', 'images/*'],
    'jspdf': ['dist/*'],
    'jszip': ['dist/*'],
    'moment': ['min/*'],
    'moment-range': ['dist/*'],
    'pdfmake': ['build/*'],
}

STATICFILES_STORAGE = 'django.contrib.staticfiles.storage.ManifestStaticFilesStorage'

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = '/static/'
MEDIA_URL = '/media/'

# Used for @login_required ecosystem
LOGIN_URL = 'two_factor:login'
LOGIN_REDIRECT_URL = 'web.views.appointments'
LOGOUT_REDIRECT_URL = 'web.views.appointments'

# Used for LoginRequiredMiddleware
STRONGHOLD_PUBLIC_NAMED_URLS = (LOGIN_URL,)

from .local_settings import *

if not SERVE_STATIC:
    MIDDLEWARE.remove('whitenoise.middleware.WhiteNoiseMiddleware')

if FORCE_2FA:
    MIDDLEWARE.append('smash.middleware.force_2fa_middleware.Force2FAMiddleware')

NPM_STATIC_FILES_PREFIX = 'npm'

# ***************
# hacky way to fix problem with '"' in css files that generate errors on collectstatic django command
# ***************
original_hashed_name = storage.HashedFilesMixin.hashed_name


@functools.wraps(original_hashed_name)
def hashed_name(self, name, *args, **kwargs):
    return original_hashed_name(self, name.replace('"', ''), *args, **kwargs)


storage.HashedFilesMixin.hashed_name = hashed_name
# ***************
