# coding=utf-8
import re

from django.db import models

from web.models.constants import CANCELLED_APPOINTMENT_COLOR_CONFIGURATION_TYPE, \
    NO_SHOW_APPOINTMENT_COLOR_CONFIGURATION_TYPE, KIT_EMAIL_HOUR_CONFIGURATION_TYPE, \
    KIT_EMAIL_DAY_OF_WEEK_CONFIGURATION_TYPE, KIT_DAILY_EMAIL_TIME_FORMAT_TYPE, KIT_DAILY_EMAIL_DAYS_PERIOD_TYPE, \
    VALUE_TYPE_CHOICES, VALUE_TYPE_TEXT


class ConfigurationItem(models.Model):
    class Meta:
        app_label = 'web'

    type = models.CharField(max_length=50,
                            verbose_name='Type',
                            editable=False
                            )
    name = models.CharField(max_length=255,
                            verbose_name='Name',
                            editable=False
                            )

    value = models.CharField(max_length=1024,
                             verbose_name='Value',
                             )
    value_type = models.CharField(max_length=32,
                                  choices=VALUE_TYPE_CHOICES,
                                  verbose_name='Value type',
                                  default=VALUE_TYPE_TEXT
                                  )

    def __str__(self):
        return "%s %s" % (self.name, self.value)

    @staticmethod
    def is_valid(item):
        message = ConfigurationItem.validation_error(item)
        return message == ""

    @staticmethod
    def validation_error(item):
        pattern = None
        if item.type == CANCELLED_APPOINTMENT_COLOR_CONFIGURATION_TYPE \
                or item.type == NO_SHOW_APPOINTMENT_COLOR_CONFIGURATION_TYPE:
            pattern = "^#[0-9a-fA-F]+$"
        if item.type == KIT_EMAIL_HOUR_CONFIGURATION_TYPE:
            pattern = "^[0-9]{2}:[0-9]{2}$"
        if item.type == KIT_DAILY_EMAIL_DAYS_PERIOD_TYPE:
            pattern = "^[0-9]+$"
        if item.type == KIT_EMAIL_DAY_OF_WEEK_CONFIGURATION_TYPE:
            pattern = "^(MONDAY|TUESDAY|WEDNESDAY|THURSDAY|FRIDAY|SATURDAY|SUNDAY)$"
        if item.type == KIT_DAILY_EMAIL_TIME_FORMAT_TYPE:
            pattern = "^(%Y-%m-%d|%Y-%m-%d %H:%M)$"
        if pattern is not None:
            if not re.compile(pattern).match(item.value):
                return "Invalid value of param: " + item.name + ". It should match regex pattern: " + pattern

        return ""
