# coding=utf-8
import locale

from django.core.files.storage import FileSystemStorage
from django.conf import settings

BOOL_CHOICES = ((True, 'Yes'), (False, 'No'))
SEX_CHOICES_MALE = 'M'
SEX_CHOICES_FEMALE = 'F'
SEX_CHOICES = (
    (SEX_CHOICES_MALE, 'Male'),
    (SEX_CHOICES_FEMALE, 'Female'),
)
BOOL_CHOICES_WITH_NONE = (
    (True, 'Yes'),
    (False, 'No'),
    (None, 'N/A'),
)

VALUE_TYPE_PASSWORD = 'PASSWORD'
VALUE_TYPE_TEXT = 'TEXT'
VALUE_TYPE_CHOICES = (
    (VALUE_TYPE_PASSWORD, 'Password'),
    (VALUE_TYPE_TEXT, 'Text'),
)


SUBJECT_TYPE_CHOICES_CONTROL = 'C'
SUBJECT_TYPE_CHOICES_PATIENT = 'P'
SUBJECT_TYPE_CHOICES = {
    SUBJECT_TYPE_CHOICES_CONTROL: 'CONTROL',
    SUBJECT_TYPE_CHOICES_PATIENT: 'PATIENT',
}
SCREENING_NUMBER_PREFIXES_FOR_TYPE = {
    SUBJECT_TYPE_CHOICES_CONTROL: "L",
    SUBJECT_TYPE_CHOICES_PATIENT: "P",
}

APPOINTMENT_TYPE_DEFAULT_COLOR = '#cfc600'
APPOINTMENT_TYPE_DEFAULT_FONT_COLOR = '#00000'

CONTACT_TYPES_EMAIL = 'E'
CONTACT_TYPES_PHONE = 'P'
CONTACT_TYPES_POST_MAIL = 'M'
CONTACT_TYPES_SMS = 'S'
CONTACT_TYPES_FAX = 'X'
CONTACT_TYPES_FACE2FACE = 'F'

CONTACT_TYPES_CHOICES = (
    (CONTACT_TYPES_EMAIL, 'Email'),
    (CONTACT_TYPES_POST_MAIL, 'Post mail'),
    (CONTACT_TYPES_FACE2FACE, 'Face to face'),
    (CONTACT_TYPES_FAX, 'Fax'),
    (CONTACT_TYPES_PHONE, 'Phone'),
    (CONTACT_TYPES_SMS, 'SMS'),
)

VISIT_SHOW_VISIT_NUMBER_FROM_ZERO = "VISIT_SHOW_VISIT_NUMBER_FROM_ZERO"

CANCELLED_APPOINTMENT_COLOR_CONFIGURATION_TYPE = "CANCELLED_APPOINTMENT_COLOR"
NO_SHOW_APPOINTMENT_COLOR_CONFIGURATION_TYPE = "NO_SHOW_APPOINTMENT_COLOR"

KIT_RECIPIENT_EMAIL_CONFIGURATION_TYPE = "KIT_RECIPIENT_EMAIL_CONFIGURATION_TYPE"
KIT_EMAIL_HOUR_CONFIGURATION_TYPE = "KIT_DAILY_EMAIL_HOUR_CONFIGURATION_TYPE"
KIT_EMAIL_DAY_OF_WEEK_CONFIGURATION_TYPE = "KIT_EMAIL_DAY_OF_WEEK_CONFIGURATION_TYPE"
KIT_DAILY_EMAIL_DAYS_PERIOD_TYPE = "KIT_DAILY_EMAIL_DAYS_PERIOD_TYPE"
KIT_DAILY_EMAIL_TIME_FORMAT_TYPE = "KIT_DAILY_EMAIL_TIME_FORMAT_TYPE"
VIRUS_EMAIL_HOUR_CONFIGURATION_TYPE = "VIRUS_EMAIL_HOUR_CONFIGURATION_TYPE"
DEFAULT_FROM_EMAIL = "DEFAULT_FROM_EMAIL"
IMPORTER_USER = "IMPORTER_USER"
IMPORT_APPOINTMENT_TYPE = "IMPORT_APPOINTMENT_TYPE"
NEXMO_API_KEY = "NEXMO_API_KEY"
NEXMO_API_SECRET = "NEXMO_API_SECRET"
NEXMO_DEFAULT_FROM = "NEXMO_DEFAULT_FROM"
LOGIN_PAGE_BACKGROUND_IMAGE = "LOGIN_PAGE_BACKGROUND_IMAGE"

RED_CAP_LANGUAGE_4_FIELD_TYPE = 'RED_CAP_LANGUAGE_4_FIELD_TYPE'
RED_CAP_LANGUAGE_3_FIELD_TYPE = 'RED_CAP_LANGUAGE_3_FIELD_TYPE'
RED_CAP_LANGUAGE_2_FIELD_TYPE = 'RED_CAP_LANGUAGE_2_FIELD_TYPE'
RED_CAP_LANGUAGE_1_FIELD_TYPE = 'RED_CAP_LANGUAGE_1_FIELD_TYPE'
RED_CAP_MPOWER_ID_FIELD_TYPE = 'RED_CAP_MPOWER_ID_FIELD_TYPE'
RED_CAP_DEAD_FIELD_TYPE = 'RED_CAP_DEAD_FIELD_TYPE'
RED_CAP_SEX_FIELD_TYPE = 'RED_CAP_SEX_FIELD_TYPE'
RED_CAP_DATE_BORN_FIELD_TYPE = 'RED_CAP_DATE_BORN_FIELD_TYPE'
RED_CAP_ND_NUMBER_FIELD_TYPE = 'RED_CAP_ND_NUMBER_FIELD_TYPE'
RED_CAP_VIRUS_FIELD_TYPE = 'RED_CAP_VIRUS_FIELD_TYPE'
RED_CAP_KIT_ID_FIELD_TYPE = "RED_CAP_KIT_ID_FIELD_TYPE"
RED_CAP_SAMPLE_DATE_FIELD_TYPE = "RED_CAP_SAMPLE_DATE_FIELD_TYPE"
RED_CAP_IGA_STATUS_FIELD_TYPE = "RED_CAP_IGA_STATUS_FIELD_TYPE"
RED_CAP_IGG_STATUS_FIELD_TYPE = "RED_CAP_IGG_STATUS_FIELD_TYPE"

MAIL_TEMPLATE_CONTEXT_SUBJECT = 'S'
MAIL_TEMPLATE_CONTEXT_APPOINTMENT = 'A'
MAIL_TEMPLATE_CONTEXT_VISIT = 'V'
MAIL_TEMPLATE_CONTEXT_VOUCHER = 'C'

MAIL_TEMPLATE_CONTEXT_CHOICES = (
    (MAIL_TEMPLATE_CONTEXT_APPOINTMENT, 'Appointment'),
    (MAIL_TEMPLATE_CONTEXT_SUBJECT, 'Subject'),
    (MAIL_TEMPLATE_CONTEXT_VISIT, 'Visit'),
    (MAIL_TEMPLATE_CONTEXT_VOUCHER, 'Voucher'),
)
LOCALE_CHOICES = [(value, value) for value in sorted(locale.windows_locale.values())]

DEFAULT_LOCALE_NAME = "fr_FR"

MONDAY_AS_DAY_OF_WEEK = 1
TUESDAY_AS_DAY_OF_WEEK = 2
WEDNESDAY_AS_DAY_OF_WEEK = 3
THURSDAY_AS_DAY_OF_WEEK = 4
FRIDAY_AS_DAY_OF_WEEK = 5
SATURDAY_AS_DAY_OF_WEEK = 6
SUNDAY_AS_DAY_OF_WEEK = 7

WEEKDAY_CHOICES = (
    (MONDAY_AS_DAY_OF_WEEK, 'MONDAY'),
    (TUESDAY_AS_DAY_OF_WEEK, 'TUESDAY'),
    (WEDNESDAY_AS_DAY_OF_WEEK, 'WEDNESDAY'),
    (THURSDAY_AS_DAY_OF_WEEK, 'THURSDAY'),
    (FRIDAY_AS_DAY_OF_WEEK, 'FRIDAY'),
    (SATURDAY_AS_DAY_OF_WEEK, 'SATURDAY'),
    (SUNDAY_AS_DAY_OF_WEEK, 'SUNDAY'),
)

AVAILABILITY_HOLIDAY = 'H'
AVAILABILITY_EXTRA = 'X'

AVAILABILITY_CHOICES = (
    (AVAILABILITY_HOLIDAY, 'Holiday'),
    (AVAILABILITY_EXTRA, 'Extra Availability'),
)

REDCAP_TOKEN_CONFIGURATION_TYPE = "REDCAP_TOKEN_CONFIGURATION_TYPE"
REDCAP_BASE_URL_CONFIGURATION_TYPE = "REDCAP_BASE_URL_CONFIGURATION_TYPE"

# timeout job after 15 minutes
CRON_JOB_TIMEOUT = 15 * 60

COUNTRY_OTHER_ID = 1
COUNTRY_AFGHANISTAN_ID = 2

# id of the singleton Study,
# TODO remove after allowing many studies per Smasch instance
GLOBAL_STUDY_ID = 1

VOUCHER_STATUS_NEW = "NEW"
VOUCHER_STATUS_IN_USE = "IN_USE"
VOUCHER_STATUS_USED = "USED"
VOUCHER_STATUS_EXPIRED = "EXPIRED"
VOUCHER_STATUS_REMOVED = "REMOVED"
VOUCHER_STATUS_CHOICES = (
    (VOUCHER_STATUS_NEW, 'New'),
    (VOUCHER_STATUS_IN_USE, 'In use'),
    (VOUCHER_STATUS_USED, 'Used'),
    (VOUCHER_STATUS_EXPIRED, 'Expired'),
    (VOUCHER_STATUS_REMOVED, 'Removed'),
)

FILE_STORAGE = FileSystemStorage(location=settings.UPLOAD_ROOT)

CUSTOM_FIELD_TYPE_TEXT = "TEXT"
CUSTOM_FIELD_TYPE_BOOLEAN = "BOOL"
CUSTOM_FIELD_TYPE_INTEGER = "INTEGER"
CUSTOM_FIELD_TYPE_DOUBLE = "DOUBLE"
CUSTOM_FIELD_TYPE_DATE = "DATE"
CUSTOM_FIELD_TYPE_SELECT_LIST = "SELECT_LIST"
CUSTOM_FIELD_TYPE_FILE = "FILE"
CUSTOM_FIELD_TYPE = (
    (CUSTOM_FIELD_TYPE_TEXT, "Text"),
    (CUSTOM_FIELD_TYPE_BOOLEAN, "Boolean (True/False)"),
    (CUSTOM_FIELD_TYPE_INTEGER, "Integer"),
    (CUSTOM_FIELD_TYPE_DOUBLE, "Double (real number)"),
    (CUSTOM_FIELD_TYPE_DATE, "Date"),
    (CUSTOM_FIELD_TYPE_SELECT_LIST, "Select list"),
    (CUSTOM_FIELD_TYPE_FILE, "File"),
)
