import logging
import os
from django.conf import settings
from django.contrib.auth.models import User
from django.test import Client
from django.test import TestCase
from web.models import Worker
from web.decorators import PermissionDecorator

from .functions import create_worker, create_user, add_permissions_to_worker

settings.MEDIA_ROOT = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')

logger = logging.getLogger(__name__)


class LoggedInTestCase(TestCase):
    def setUp(self):
        self.password = 'abcd1234'
        #superuser
        self.super_worker = create_worker(user=User.objects.create_superuser(username='super', password=self.password, email='a@mail.com'))
        #admin 
        self.admin_worker = Worker.get_by_user(create_user(username='admin', password=self.password))
        add_permissions_to_worker(self.admin_worker, PermissionDecorator.codenames)
        #staff
        self.staff_worker = Worker.get_by_user(create_user(username='staff', password=self.password))

        self.client = Client()
        username = 'piotr'
        password = 'top_secret'
        self.user = User.objects.create_user(
            username=username, email='jacob@bla', password=password)
        self.client.login(username=username, password=password)

    def login_as_staff(self):
        self.client.logout()
        self.client.login(username='staff', password=self.password) 

    def login_as_admin(self):
        self.client.logout()
        self.client.login(username='admin', password=self.password) 

    def login_as_super(self):
        self.client.logout()
        self.client.login(username='super', password=self.password) 


class LoggedInWithWorkerTestCase(LoggedInTestCase):
    def setUp(self):
        super(LoggedInWithWorkerTestCase, self).setUp()
        self.worker = create_worker(self.user, True)
