from django.test import TestCase

from web.models import ConfigurationItem
from web.models.constants import CANCELLED_APPOINTMENT_COLOR_CONFIGURATION_TYPE, \
    NO_SHOW_APPOINTMENT_COLOR_CONFIGURATION_TYPE, KIT_EMAIL_HOUR_CONFIGURATION_TYPE, \
    KIT_EMAIL_DAY_OF_WEEK_CONFIGURATION_TYPE
from web.tests.functions import create_configuration_item


class ConfigurationItemModelTests(TestCase):
    def test_init_data(self):
        items = ConfigurationItem.objects.filter(type=CANCELLED_APPOINTMENT_COLOR_CONFIGURATION_TYPE)
        self.assertTrue(len(items) > 0)

        items = ConfigurationItem.objects.filter(type=NO_SHOW_APPOINTMENT_COLOR_CONFIGURATION_TYPE)
        self.assertTrue(len(items) > 0)

    def test_str(self):
        configuration_item = create_configuration_item()

        self.assertIsNotNone(str(configuration_item))
        self.assertIsNotNone(str(configuration_item))

    def test_validate(self):
        item = ConfigurationItem.objects.filter(type=KIT_EMAIL_HOUR_CONFIGURATION_TYPE)[0]
        item.value = "09:00"
        self.assertEqual("", ConfigurationItem.validation_error(item))
        item.value = "text"
        self.assertNotEqual("", ConfigurationItem.validation_error(item))

        item = ConfigurationItem.objects.filter(type=KIT_EMAIL_DAY_OF_WEEK_CONFIGURATION_TYPE)[0]
        item.value = "MONDAY"
        self.assertEqual("", ConfigurationItem.validation_error(item))
        item.value = "unknown day"
        self.assertNotEqual("", ConfigurationItem.validation_error(item))
